@@ -15,19 +15,27 @@
 
 import functools
 
+import netaddr
+
+from neutron_lib.api.definitions import network as net_def
+from neutron_lib.api.definitions import port as port_def
+from neutron_lib.api.definitions import subnet as subnet_def
+from neutron_lib.api.definitions import subnetpool as subnetpool_def
 from neutron_lib.api import validators
 from neutron_lib import constants
-from neutron_lib import exceptions as n_exc
+from neutron_lib.db import api as db_api
+from neutron_lib.db import model_query
+from neutron_lib.db import resource_extend
+from neutron_lib.db import utils as db_utils
+from neutron_lib import exceptions
+from neutron_lib.utils import net
 from oslo_config import cfg
 from oslo_log import log as logging
 from sqlalchemy.orm import exc
 
-from neutron.api.v2 import attributes
-from neutron.common import constants as n_const
-from neutron.common import exceptions
-from neutron.common import utils
-from neutron.db import common_db_mixin
 from neutron.db import models_v2
+from neutron.objects import base as base_obj
+from neutron.objects import ports as port_obj
 from neutron.objects import subnet as subnet_obj
 from neutron.objects import subnetpool as subnetpool_obj
 
@@ -68,7 +76,22 @@ def filter_fields(f):
     return inner_filter
 
 
-class DbBasePluginCommon(common_db_mixin.CommonDbMixin):
+def make_result_with_fields(f):
+    @functools.wraps(f)
+    def inner(*args, **kwargs):
+        fields = kwargs.get('fields')
+        result = f(*args, **kwargs)
+        if fields is None:
+            return result
+        elif isinstance(result, list):
+            return [db_utils.resource_fields(r, fields) for r in result]
+        else:
+            return db_utils.resource_fields(result, fields)
+
+    return inner
+
+
+class DbBasePluginCommon(object):
     """Stores getters and helper methods for db_base_plugin_v2
 
     All private getters and simple helpers like _make_*_dict were moved from
@@ -79,14 +102,14 @@ class DbBasePluginCommon(common_db_mixin.CommonDbMixin):
     """
 
     @staticmethod
-    def _generate_mac():
-        return utils.get_random_mac(cfg.CONF.base_mac.split(':'))
+    def _generate_macs(mac_count=1):
+        mac_maker = net.random_mac_generator(cfg.CONF.base_mac.split(':'))
+        return [next(mac_maker) for x in range(mac_count)]
 
+    @db_api.CONTEXT_READER
     def _is_mac_in_use(self, context, network_id, mac_address):
-        return bool(context.session.query(models_v2.Port).
-                    filter(models_v2.Port.network_id == network_id).
-                    filter(models_v2.Port.mac_address == mac_address).
-                    count())
+        return port_obj.Port.objects_exist(context, network_id=network_id,
+                                           mac_address=mac_address)
 
     @staticmethod
     def _delete_ip_allocation(context, network_id, subnet_id, ip_address):
@@ -97,14 +120,12 @@ class DbBasePluginCommon(common_db_mixin.CommonDbMixin):
                   {'ip_address': ip_address,
                    'network_id': network_id,
                    'subnet_id': subnet_id})
-        with context.session.begin(subtransactions=True):
-            for ipal in (context.session.query(models_v2.IPAllocation).
-                         filter_by(network_id=network_id,
-                                   ip_address=ip_address,
-                                   subnet_id=subnet_id)):
-                context.session.delete(ipal)
+        port_obj.IPAllocation.delete_objects(
+            context, network_id=network_id, ip_address=ip_address,
+            subnet_id=subnet_id)
 
     @staticmethod
+    @db_api.CONTEXT_WRITER
     def _store_ip_allocation(context, ip_address, network_id, subnet_id,
                              port_id):
         LOG.debug("Allocated IP %(ip_address)s "
@@ -113,20 +134,13 @@ class DbBasePluginCommon(common_db_mixin.CommonDbMixin):
                    'network_id': network_id,
                    'subnet_id': subnet_id,
                    'port_id': port_id})
-        allocated = models_v2.IPAllocation(
-            network_id=network_id,
-            port_id=port_id,
-            ip_address=ip_address,
-            subnet_id=subnet_id
-        )
-        context.session.add(allocated)
-
-        # NOTE(kevinbenton): We add this to the session info so the sqlalchemy
-        # object isn't immediately garbage collected. Otherwise when the
-        # fixed_ips relationship is referenced a new persistent object will be
-        # added to the session that will interfere with retry operations.
-        # See bug 1556178 for details.
-        context.session.info.setdefault('allocated_ips', []).append(allocated)
+        allocated = port_obj.IPAllocation(
+            context, network_id=network_id, port_id=port_id,
+            ip_address=ip_address, subnet_id=subnet_id)
+        # NOTE(lujinluo): Add IPAllocations obj to the port fixed_ips
+        # in Port OVO integration, i.e. the same way we did in
+        # Ib32509d974c8654131112234bcf19d6eae8f7cca
+        allocated.create()
 
     def _make_subnet_dict(self, subnet, fields=None, context=None):
         res = {'id': subnet['id'],
@@ -134,26 +148,51 @@ class DbBasePluginCommon(common_db_mixin.CommonDbMixin):
                'tenant_id': subnet['tenant_id'],
                'network_id': subnet['network_id'],
                'ip_version': subnet['ip_version'],
-               'cidr': subnet['cidr'],
-               'subnetpool_id': subnet.get('subnetpool_id'),
-               'allocation_pools': [{'start': pool['first_ip'],
-                                     'end': pool['last_ip']}
-                                    for pool in subnet['allocation_pools']],
-               'gateway_ip': subnet['gateway_ip'],
+               'subnetpool_id': subnet['subnetpool_id'],
                'enable_dhcp': subnet['enable_dhcp'],
                'ipv6_ra_mode': subnet['ipv6_ra_mode'],
                'ipv6_address_mode': subnet['ipv6_address_mode'],
-               'dns_nameservers': [dns['address']
-                                   for dns in subnet['dns_nameservers']],
-               'host_routes': [{'destination': route['destination'],
-                                'nexthop': route['nexthop']}
-                               for route in subnet['routes']],
                }
-        # The shared attribute for a subnet is the same as its parent network
-        res['shared'] = self._is_network_shared(context, subnet.networks)
-        # Call auxiliary extend functions, if any
-        self._apply_dict_extend_functions(attributes.SUBNETS, res, subnet)
-        return self._fields(res, fields)
+        res['gateway_ip'] = str(
+                subnet['gateway_ip']) if subnet['gateway_ip'] else None
+        # TODO(korzen) this method can get subnet as DB object or Subnet OVO,
+        # so temporary workaround will be to fill in the fields in separate
+        # ways. After converting all code pieces to use Subnet OVO, the latter
+        # 'else' can be deleted
+        if isinstance(subnet, subnet_obj.Subnet):
+            res['cidr'] = str(subnet.cidr)
+            res['allocation_pools'] = [{'start': str(pool.start),
+                                       'end': str(pool.end)}
+                                       for pool in subnet.allocation_pools]
+            res['host_routes'] = [{'destination': str(route.destination),
+                                   'nexthop': str(route.nexthop)}
+                                  for route in subnet.host_routes]
+            res['dns_nameservers'] = [str(dns.address)
+                                      for dns in subnet.dns_nameservers]
+            res['shared'] = subnet.shared
+            # Call auxiliary extend functions, if any
+            resource_extend.apply_funcs(subnet_def.COLLECTION_NAME,
+                                        res, subnet.db_obj)
+        else:
+            res['cidr'] = subnet['cidr']
+            res['allocation_pools'] = [{'start': pool['first_ip'],
+                                       'end': pool['last_ip']}
+                                       for pool in subnet['allocation_pools']]
+            res['host_routes'] = [{'destination': route['destination'],
+                                   'nexthop': route['nexthop']}
+                                  for route in subnet['routes']]
+            res['dns_nameservers'] = [dns['address']
+                                      for dns in subnet['dns_nameservers']]
+
+            # The shared attribute for a subnet is the same
+            # as its parent network
+            res['shared'] = self._is_network_shared(context,
+                                                    subnet.rbac_entries)
+            # Call auxiliary extend functions, if any
+            resource_extend.apply_funcs(subnet_def.COLLECTION_NAME,
+                                        res, subnet)
+
+        return db_utils.resource_fields(res, fields)
 
     def _make_subnetpool_dict(self, subnetpool, fields=None):
         default_prefixlen = str(subnetpool['default_prefixlen'])
@@ -167,46 +206,55 @@ class DbBasePluginCommon(common_db_mixin.CommonDbMixin):
                'max_prefixlen': max_prefixlen,
                'is_default': subnetpool['is_default'],
                'shared': subnetpool['shared'],
-               'prefixes': [prefix.cidr for prefix in subnetpool['prefixes']],
+               'prefixes': [str(prefix.cidr)
+                            for prefix in subnetpool['prefixes']],
                'ip_version': subnetpool['ip_version'],
                'default_quota': subnetpool['default_quota'],
                'address_scope_id': subnetpool['address_scope_id']}
-        self._apply_dict_extend_functions(attributes.SUBNETPOOLS, res,
-                                          subnetpool)
-        return self._fields(res, fields)
+        resource_extend.apply_funcs(
+            subnetpool_def.COLLECTION_NAME, res, subnetpool.db_obj)
+        return db_utils.resource_fields(res, fields)
 
     def _make_port_dict(self, port, fields=None,
-                        process_extensions=True):
+                        process_extensions=True,
+                        with_fixed_ips=True):
+        mac = port["mac_address"]
+        if isinstance(mac, netaddr.EUI):
+            mac.dialect = netaddr.mac_unix_expanded
         res = {"id": port["id"],
                'name': port['name'],
                "network_id": port["network_id"],
                'tenant_id': port['tenant_id'],
-               "mac_address": port["mac_address"],
+               "mac_address": str(mac),
                "admin_state_up": port["admin_state_up"],
                "status": port["status"],
-               "fixed_ips": [{'subnet_id': ip["subnet_id"],
-                              'ip_address': ip["ip_address"]}
-                             for ip in port["fixed_ips"]],
                "device_id": port["device_id"],
                "device_owner": port["device_owner"]}
+        if with_fixed_ips:
+            res["fixed_ips"] = [
+                {'subnet_id': ip["subnet_id"],
+                 'ip_address': str(
+                     ip["ip_address"])} for ip in port["fixed_ips"]]
         # Call auxiliary extend functions, if any
         if process_extensions:
-            self._apply_dict_extend_functions(
-                attributes.PORTS, res, port)
-        return self._fields(res, fields)
+            port_data = port
+            if isinstance(port, port_obj.Port):
+                port_data = port.db_obj
+            resource_extend.apply_funcs(
+                port_def.COLLECTION_NAME, res, port_data)
+        return db_utils.resource_fields(res, fields)
 
     def _get_network(self, context, id):
         try:
-            network = self._get_by_id(context, models_v2.Network, id)
+            network = model_query.get_by_id(context, models_v2.Network, id)
         except exc.NoResultFound:
-            raise n_exc.NetworkNotFound(net_id=id)
+            raise exceptions.NetworkNotFound(net_id=id)
         return network
 
-    def _get_subnet(self, context, id):
-        try:
-            subnet = self._get_by_id(context, models_v2.Subnet, id)
-        except exc.NoResultFound:
-            raise n_exc.SubnetNotFound(subnet_id=id)
+    def _get_subnet_object(self, context, id):
+        subnet = subnet_obj.Subnet.get_object(context, id=id)
+        if not subnet:
+            raise exceptions.SubnetNotFound(subnet_id=id)
         return subnet
 
     def _get_subnetpool(self, context, id):
@@ -218,50 +266,40 @@ class DbBasePluginCommon(common_db_mixin.CommonDbMixin):
 
     def _get_port(self, context, id):
         try:
-            port = self._get_by_id(context, models_v2.Port, id)
+            port = model_query.get_by_id(context, models_v2.Port, id)
         except exc.NoResultFound:
-            raise n_exc.PortNotFound(port_id=id)
+            raise exceptions.PortNotFound(port_id=id)
         return port
 
-    def _get_dns_by_subnet(self, context, subnet_id):
-        return subnet_obj.DNSNameServer.get_objects(context,
-                                                    subnet_id=subnet_id)
-
     def _get_route_by_subnet(self, context, subnet_id):
-        route_qry = context.session.query(models_v2.SubnetRoute)
-        return route_qry.filter_by(subnet_id=subnet_id).all()
+        return subnet_obj.Route.get_objects(context,
+                                            subnet_id=subnet_id)
 
     def _get_router_gw_ports_by_network(self, context, network_id):
-        port_qry = context.session.query(models_v2.Port)
-        return port_qry.filter_by(network_id=network_id,
-                device_owner=constants.DEVICE_OWNER_ROUTER_GW).all()
+        return port_obj.Port.get_objects(
+            context, network_id=network_id,
+            device_owner=constants.DEVICE_OWNER_ROUTER_GW)
 
+    @db_api.CONTEXT_READER
     def _get_subnets_by_network(self, context, network_id):
-        subnet_qry = context.session.query(models_v2.Subnet)
-        return subnet_qry.filter_by(network_id=network_id).all()
+        return subnet_obj.Subnet.get_objects(context, network_id=network_id)
 
+    @db_api.CONTEXT_READER
     def _get_subnets_by_subnetpool(self, context, subnetpool_id):
-        subnet_qry = context.session.query(models_v2.Subnet)
-        return subnet_qry.filter_by(subnetpool_id=subnetpool_id).all()
+        return subnet_obj.Subnet.get_objects(context,
+                                             subnetpool_id=subnetpool_id)
 
-    def _get_all_subnets(self, context):
-        # NOTE(salvatore-orlando): This query might end up putting
-        # a lot of stress on the db. Consider adding a cache layer
-        return context.session.query(models_v2.Subnet).all()
-
-    def _get_subnets(self, context, filters=None, fields=None,
-                     sorts=None, limit=None, marker=None,
-                     page_reverse=False):
-        marker_obj = self._get_marker_obj(context, 'subnet', limit, marker)
-        make_subnet_dict = functools.partial(self._make_subnet_dict,
-                                             context=context)
-        return self._get_collection(context, models_v2.Subnet,
-                                    make_subnet_dict,
-                                    filters=filters, fields=fields,
-                                    sorts=sorts,
-                                    limit=limit,
-                                    marker_obj=marker_obj,
-                                    page_reverse=page_reverse)
+    def _get_subnets(self, context, filters=None, sorts=None, limit=None,
+                     marker=None, page_reverse=False):
+        pager = base_obj.Pager(sorts, limit, page_reverse, marker)
+        filters = filters or {}
+        # turn the CIDRs into a proper subnets
+        if filters.get('cidr'):
+            filters.update(
+                {'cidr': [netaddr.IPNetwork(x).cidr for x in filters['cidr']]})
+        return subnet_obj.Subnet.get_objects(context, _pager=pager,
+                                             validate_filters=False,
+                                             **filters)
 
     def _make_network_dict(self, network, fields=None,
                            process_extensions=True, context=None):
@@ -269,38 +307,36 @@ class DbBasePluginCommon(common_db_mixin.CommonDbMixin):
                'name': network['name'],
                'tenant_id': network['tenant_id'],
                'admin_state_up': network['admin_state_up'],
-               'mtu': network.get('mtu', n_const.DEFAULT_NETWORK_MTU),
+               'mtu': network.get('mtu', constants.DEFAULT_NETWORK_MTU),
                'status': network['status'],
                'subnets': [subnet['id']
                            for subnet in network['subnets']]}
-        res['shared'] = self._is_network_shared(context, network)
+        res['shared'] = self._is_network_shared(context, network.rbac_entries)
         # Call auxiliary extend functions, if any
         if process_extensions:
-            self._apply_dict_extend_functions(
-                attributes.NETWORKS, res, network)
-        return self._fields(res, fields)
+            resource_extend.apply_funcs(net_def.COLLECTION_NAME, res, network)
+        return db_utils.resource_fields(res, fields)
 
-    def _is_network_shared(self, context, network):
+    def _is_network_shared(self, context, rbac_entries):
         # The shared attribute for a network now reflects if the network
         # is shared to the calling tenant via an RBAC entry.
         matches = ('*',) + ((context.tenant_id,) if context else ())
-        for entry in network.rbac_entries:
+        for entry in rbac_entries:
             if (entry.action == 'access_as_shared' and
                     entry.target_tenant in matches):
                 return True
         return False
 
     def _make_subnet_args(self, detail, subnet, subnetpool_id):
-        gateway_ip = str(detail.gateway_ip) if detail.gateway_ip else None
-        args = {'tenant_id': detail.tenant_id,
+        args = {'project_id': detail.tenant_id,
                 'id': detail.subnet_id,
                 'name': subnet['name'],
                 'network_id': subnet['network_id'],
                 'ip_version': subnet['ip_version'],
-                'cidr': str(detail.subnet_cidr),
+                'cidr': detail.subnet_cidr,
                 'subnetpool_id': subnetpool_id,
                 'enable_dhcp': subnet['enable_dhcp'],
-                'gateway_ip': gateway_ip,
+                'gateway_ip': detail.gateway_ip,
                 'description': subnet.get('description')}
         if subnet['ip_version'] == 6 and subnet['enable_dhcp']:
             if validators.is_attr_set(subnet['ipv6_ra_mode']):
@@ -308,27 +344,3 @@ class DbBasePluginCommon(common_db_mixin.CommonDbMixin):
             if validators.is_attr_set(subnet['ipv6_address_mode']):
                 args['ipv6_address_mode'] = subnet['ipv6_address_mode']
         return args
-
-    def _make_fixed_ip_dict(self, ips):
-        # Excludes from dict all keys except subnet_id and ip_address
-        return [{'subnet_id': ip["subnet_id"],
-                 'ip_address': ip["ip_address"]}
-                for ip in ips]
-
-    def _port_filter_hook(self, context, original_model, conditions):
-        # Apply the port filter only in non-admin and non-advsvc context
-        if self.model_query_scope(context, original_model):
-            conditions |= (
-                (context.tenant_id == models_v2.Network.tenant_id) &
-                (models_v2.Network.id == models_v2.Port.network_id))
-        return conditions
-
-    def _port_query_hook(self, context, original_model, query):
-        # we need to outerjoin to networks if the model query scope
-        # is necessary so we can filter based on network id. without
-        # this the conditions in the filter hook cause the networks
-        # table to be added to the FROM statement so we get lots of
-        # duplicated rows that break the COUNT operation
-        if self.model_query_scope(context, original_model):
-            query = query.outerjoin(models_v2.Network)
-        return query
