@@ -18,13 +18,13 @@ import netaddr
 from designateclient import exceptions as d_exc
 from designateclient.v2 import client as d_client
 from keystoneauth1.identity.generic import password
-from keystoneauth1 import session
+from keystoneauth1 import loading
 from keystoneauth1 import token_endpoint
 from neutron_lib import constants
+from neutron_lib.exceptions import dns as dns_exc
 from oslo_config import cfg
 
 from neutron.conf.services import extdns_designate_driver
-from neutron.extensions import dns
 from neutron.services.externaldns import driver
 
 IPV4_PTR_ZONE_PREFIX_MIN_SIZE = 8
@@ -42,24 +42,31 @@ def get_clients(context):
     global _SESSION
 
     if not _SESSION:
-        if CONF.designate.insecure:
-            verify = False
-        else:
-            verify = CONF.designate.ca_cert or True
-        _SESSION = session.Session(verify=verify)
+        _SESSION = loading.load_session_from_conf_options(
+            CONF, 'designate')
 
     auth = token_endpoint.Token(CONF.designate.url, context.auth_token)
     client = d_client.Client(session=_SESSION, auth=auth)
-    admin_auth = password.Password(
-        auth_url=CONF.designate.admin_auth_url,
-        username=CONF.designate.admin_username,
-        password=CONF.designate.admin_password,
-        tenant_name=CONF.designate.admin_tenant_name,
-        tenant_id=CONF.designate.admin_tenant_id)
-    admin_client = d_client.Client(session=_SESSION, auth=admin_auth)
+    if CONF.designate.auth_type:
+        admin_auth = loading.load_auth_from_conf_options(
+            CONF, 'designate')
+    else:
+        admin_auth = password.Password(
+            auth_url=CONF.designate.admin_auth_url,
+            username=CONF.designate.admin_username,
+            password=CONF.designate.admin_password,
+            tenant_name=CONF.designate.admin_tenant_name,
+            tenant_id=CONF.designate.admin_tenant_id)
+    admin_client = d_client.Client(session=_SESSION, auth=admin_auth,
+                                   endpoint_override=CONF.designate.url)
     return client, admin_client
 
 
+def get_all_projects_client(context):
+    auth = token_endpoint.Token(CONF.designate.url, context.auth_token)
+    return d_client.Client(session=_SESSION, auth=auth, all_projects=True)
+
+
 class Designate(driver.ExternalDNSService):
     """Driver for Designate."""
 
@@ -68,17 +75,17 @@ class Designate(driver.ExternalDNSService):
         ipv6_ptr_zone_size = CONF.designate.ipv6_ptr_zone_prefix_size
 
         if (ipv4_ptr_zone_size < IPV4_PTR_ZONE_PREFIX_MIN_SIZE or
-            ipv4_ptr_zone_size > IPV4_PTR_ZONE_PREFIX_MAX_SIZE or
-            (ipv4_ptr_zone_size % 8) != 0):
-            raise dns.InvalidPTRZoneConfiguration(
+                ipv4_ptr_zone_size > IPV4_PTR_ZONE_PREFIX_MAX_SIZE or
+                (ipv4_ptr_zone_size % 8) != 0):
+            raise dns_exc.InvalidPTRZoneConfiguration(
                 parameter='ipv4_ptr_zone_size', number='8',
                 maximum=str(IPV4_PTR_ZONE_PREFIX_MAX_SIZE),
                 minimum=str(IPV4_PTR_ZONE_PREFIX_MIN_SIZE))
 
         if (ipv6_ptr_zone_size < IPV6_PTR_ZONE_PREFIX_MIN_SIZE or
-            ipv6_ptr_zone_size > IPV6_PTR_ZONE_PREFIX_MAX_SIZE or
-            (ipv6_ptr_zone_size % 4) != 0):
-            raise dns.InvalidPTRZoneConfiguration(
+                ipv6_ptr_zone_size > IPV6_PTR_ZONE_PREFIX_MAX_SIZE or
+                (ipv6_ptr_zone_size % 4) != 0):
+            raise dns_exc.InvalidPTRZoneConfiguration(
                 parameter='ipv6_ptr_zone_size', number='4',
                 maximum=str(IPV6_PTR_ZONE_PREFIX_MAX_SIZE),
                 minimum=str(IPV6_PTR_ZONE_PREFIX_MIN_SIZE))
@@ -92,9 +99,9 @@ class Designate(driver.ExternalDNSService):
             if v6:
                 designate.recordsets.create(dns_domain, dns_name, 'AAAA', v6)
         except d_exc.NotFound:
-            raise dns.DNSDomainNotFound(dns_domain=dns_domain)
+            raise dns_exc.DNSDomainNotFound(dns_domain=dns_domain)
         except d_exc.Conflict:
-            raise dns.DuplicateRecordSet(dns_name=dns_name)
+            raise dns_exc.DuplicateRecordSet(dns_name=dns_name)
 
         if not CONF.designate.allow_reverse_dns_lookup:
             return
@@ -145,18 +152,25 @@ class Designate(driver.ExternalDNSService):
                     CONF.designate.ipv6_ptr_zone_prefix_size) / 4)
 
     def delete_record_set(self, context, dns_domain, dns_name, records):
-        designate, designate_admin = get_clients(context)
-        ids_to_delete = self._get_ids_ips_to_delete(
-            dns_domain, '%s.%s' % (dns_name, dns_domain), records, designate)
+        client, admin_client = get_clients(context)
+        try:
+            ids_to_delete = self._get_ids_ips_to_delete(
+                dns_domain, '%s.%s' % (dns_name, dns_domain), records, client)
+        except dns_exc.DNSDomainNotFound:
+            # Try whether we have admin powers and can see all projects
+            client = get_all_projects_client(context)
+            ids_to_delete = self._get_ids_ips_to_delete(
+                dns_domain, '%s.%s' % (dns_name, dns_domain), records, client)
+
         for _id in ids_to_delete:
-            designate.recordsets.delete(dns_domain, _id)
+            client.recordsets.delete(dns_domain, _id)
         if not CONF.designate.allow_reverse_dns_lookup:
             return
 
         for record in records:
             in_addr_name = netaddr.IPAddress(record).reverse_dns
             in_addr_zone_name = self._get_in_addr_zone_name(in_addr_name)
-            designate_admin.recordsets.delete(in_addr_zone_name, in_addr_name)
+            admin_client.recordsets.delete(in_addr_zone_name, in_addr_name)
 
     def _get_ids_ips_to_delete(self, dns_domain, name, records,
                                designate_client):
@@ -164,9 +178,9 @@ class Designate(driver.ExternalDNSService):
             recordsets = designate_client.recordsets.list(
                 dns_domain, criterion={"name": "%s" % name})
         except d_exc.NotFound:
-            raise dns.DNSDomainNotFound(dns_domain=dns_domain)
+            raise dns_exc.DNSDomainNotFound(dns_domain=dns_domain)
         ids = [rec['id'] for rec in recordsets]
-        ips = [ip for rec in recordsets for ip in rec['records']]
+        ips = [str(ip) for rec in recordsets for ip in rec['records']]
         if set(ips) != set(records):
-            raise dns.DuplicateRecordSet(dns_name=name)
+            raise dns_exc.DuplicateRecordSet(dns_name=name)
         return ids
