@@ -34,12 +34,34 @@ class EnvironmentDescription(object):
     Does the setup, as a whole, support tunneling? How about l2pop?
     """
     def __init__(self, network_type='vxlan', l2_pop=True, qos=False,
-                 mech_drivers='openvswitch,linuxbridge'):
+                 mech_drivers='openvswitch,linuxbridge',
+                 service_plugins='router', arp_responder=False,
+                 agent_down_time=75, router_scheduler=None,
+                 global_mtu=constants.DEFAULT_NETWORK_MTU,
+                 debug_iptables=False, log=False, report_bandwidths=False,
+                 has_placement=False, placement_port=None,
+                 dhcp_scheduler_class=None, ml2_extension_drivers=None):
         self.network_type = network_type
         self.l2_pop = l2_pop
         self.qos = qos
+        self.log = log
         self.network_range = None
         self.mech_drivers = mech_drivers
+        self.arp_responder = arp_responder
+        self.agent_down_time = agent_down_time
+        self.router_scheduler = router_scheduler
+        self.global_mtu = global_mtu
+        self.service_plugins = service_plugins
+        self.debug_iptables = debug_iptables
+        self.report_bandwidths = report_bandwidths
+        self.has_placement = has_placement
+        self.placement_port = placement_port
+        self.dhcp_scheduler_class = dhcp_scheduler_class
+        if self.qos:
+            self.service_plugins += ',qos'
+        if self.log:
+            self.service_plugins += ',log'
+        self.ml2_extension_drivers = ml2_extension_drivers
 
     @property
     def tunneling_enabled(self):
@@ -52,13 +74,18 @@ class HostDescription(object):
     What agents should the host spawn? What mode should each agent operate
     under?
     """
-    def __init__(self, l3_agent=False, of_interface='ovs-ofctl',
-                 ovsdb_interface='vsctl',
-                 l2_agent_type=constants.AGENT_TYPE_OVS):
+    def __init__(self, l3_agent=False, dhcp_agent=False,
+                 l2_agent_type=constants.AGENT_TYPE_OVS,
+                 firewall_driver='noop', availability_zone=None,
+                 l3_agent_mode=None,
+                 l3_agent_extensions=None):
         self.l2_agent_type = l2_agent_type
         self.l3_agent = l3_agent
-        self.of_interface = of_interface
-        self.ovsdb_interface = ovsdb_interface
+        self.dhcp_agent = dhcp_agent
+        self.firewall_driver = firewall_driver
+        self.availability_zone = availability_zone
+        self.l3_agent_mode = l3_agent_mode
+        self.l3_agent_extensions = l3_agent_extensions
 
 
 class Host(fixtures.Fixture):
@@ -76,15 +103,13 @@ class Host(fixtures.Fixture):
     and disconnects the host from other hosts.
     """
 
-    def __init__(self, env_desc, host_desc,
-                 test_name, neutron_config,
-                 central_data_bridge, central_external_bridge):
+    def __init__(self, env_desc, host_desc, test_name,
+                 neutron_config, central_bridge):
         self.env_desc = env_desc
         self.host_desc = host_desc
         self.test_name = test_name
         self.neutron_config = neutron_config
-        self.central_data_bridge = central_data_bridge
-        self.central_external_bridge = central_external_bridge
+        self.central_bridge = central_bridge
         self.host_namespace = None
         self.agents = {}
         # we need to cache already created "per network" bridges if linuxbridge
@@ -96,6 +121,8 @@ class Host(fixtures.Fixture):
 
         if self.host_desc.l2_agent_type == constants.AGENT_TYPE_OVS:
             self.setup_host_with_ovs_agent()
+        elif self.host_desc.l2_agent_type == constants.AGENT_TYPE_NIC_SWITCH:
+            self.setup_host_with_sriov_agent()
         elif self.host_desc.l2_agent_type == constants.AGENT_TYPE_LINUXBRIDGE:
             self.setup_host_with_linuxbridge_agent()
         if self.host_desc.l3_agent:
@@ -106,22 +133,31 @@ class Host(fixtures.Fixture):
                     self.neutron_config,
                     self.l3_agent_cfg_fixture))
 
+        if self.host_desc.dhcp_agent:
+            self.dhcp_agent = self.useFixture(
+                process.DhcpAgentFixture(
+                    self.env_desc, self.host_desc,
+                    self.test_name,
+                    self.neutron_config,
+                    self.dhcp_agent_cfg_fixture,
+                    namespace=self.host_namespace))
+
     def setup_host_with_ovs_agent(self):
         agent_cfg_fixture = config.OVSConfigFixture(
             self.env_desc, self.host_desc, self.neutron_config.temp_dir,
-            self.local_ip)
+            self.local_ip, test_name=self.test_name)
         self.useFixture(agent_cfg_fixture)
 
+        self.br_phys = self.useFixture(
+            net_helpers.OVSBridgeFixture(
+                agent_cfg_fixture.get_br_phys_name())).bridge
         if self.env_desc.tunneling_enabled:
             self.useFixture(
                 net_helpers.OVSBridgeFixture(
                     agent_cfg_fixture.get_br_tun_name())).bridge
-            self.connect_to_internal_network_via_tunneling()
+            self.connect_to_central_network_via_tunneling()
         else:
-            br_phys = self.useFixture(
-                net_helpers.OVSBridgeFixture(
-                    agent_cfg_fixture.get_br_phys_name())).bridge
-            self.connect_to_internal_network_via_vlans(br_phys)
+            self.connect_to_central_network_via_vlans(self.br_phys)
 
         self.ovs_agent = self.useFixture(
             process.OVSAgentFixture(
@@ -134,14 +170,27 @@ class Host(fixtures.Fixture):
                     self.env_desc, self.host_desc,
                     self.neutron_config.temp_dir,
                     self.ovs_agent.agent_cfg_fixture.get_br_int_name()))
-            br_ex = self.useFixture(
-                net_helpers.OVSBridgeFixture(
-                    self.l3_agent_cfg_fixture.get_external_bridge())).bridge
-            self.connect_to_external_network(br_ex)
+
+        if self.host_desc.dhcp_agent:
+            self.dhcp_agent_cfg_fixture = self.useFixture(
+                config.DhcpConfigFixture(
+                    self.env_desc, self.host_desc,
+                    self.neutron_config.temp_dir,
+                    self.ovs_agent.agent_cfg_fixture.get_br_int_name()))
+
+    def setup_host_with_sriov_agent(self):
+        agent_cfg_fixture = config.SRIOVConfigFixture(
+            self.env_desc, self.host_desc, self.neutron_config.temp_dir,
+            self.local_ip)
+        self.useFixture(agent_cfg_fixture)
+        self.sriov_agent = self.useFixture(
+            process.SRIOVAgentFixture(
+                self.env_desc, self.host_desc,
+                self.test_name, self.neutron_config, agent_cfg_fixture))
 
     def setup_host_with_linuxbridge_agent(self):
-        #First we need to provide connectivity for agent to prepare proper
-        #bridge mappings in agent's config:
+        # First we need to provide connectivity for agent to prepare proper
+        # bridge mappings in agent's config:
         self.host_namespace = self.useFixture(
             net_helpers.NamespaceFixture(prefix="host-")
         ).name
@@ -170,10 +219,16 @@ class Host(fixtures.Fixture):
                     self.env_desc, self.host_desc,
                     self.neutron_config.temp_dir))
 
+        if self.host_desc.dhcp_agent:
+            self.dhcp_agent_cfg_fixture = self.useFixture(
+                config.DhcpConfigFixture(
+                    self.env_desc, self.host_desc,
+                    self.neutron_config.temp_dir))
+
     def _connect_ovs_port(self, cidr_address):
         ovs_device = self.useFixture(
             net_helpers.OVSPortFixture(
-                bridge=self.central_data_bridge,
+                bridge=self.central_bridge,
                 namespace=self.host_namespace)).port
         # NOTE: This sets an IP address on the host's root namespace
         # which is cleaned up when the device is deleted.
@@ -186,7 +241,7 @@ class Host(fixtures.Fixture):
         )
         self.host_port.link.set_up()
 
-    def connect_to_internal_network_via_tunneling(self):
+    def connect_to_central_network_via_tunneling(self):
         veth_1, veth_2 = self.useFixture(
             net_helpers.VethFixture()).ports
 
@@ -197,15 +252,11 @@ class Host(fixtures.Fixture):
         veth_1.link.set_up()
         veth_2.link.set_up()
 
-    def connect_to_internal_network_via_vlans(self, host_data_bridge):
+    def connect_to_central_network_via_vlans(self, host_data_bridge):
         # If using VLANs as a segmentation device, it's needed to connect
         # a provider bridge to a centralized, shared bridge.
         net_helpers.create_patch_ports(
-            self.central_data_bridge, host_data_bridge)
-
-    def connect_to_external_network(self, host_external_bridge):
-        net_helpers.create_patch_ports(
-            self.central_external_bridge, host_external_bridge)
+            self.central_bridge, host_data_bridge)
 
     def allocate_local_ip(self):
         if not self.env_desc.network_range:
@@ -245,6 +296,14 @@ class Host(fixtures.Fixture):
     def l3_agent(self, agent):
         self.agents['l3'] = agent
 
+    @property
+    def dhcp_agent(self):
+        return self.agents['dhcp']
+
+    @dhcp_agent.setter
+    def dhcp_agent(self, agent):
+        self.agents['dhcp'] = agent
+
     @property
     def ovs_agent(self):
         return self.agents['ovs']
@@ -253,6 +312,14 @@ class Host(fixtures.Fixture):
     def ovs_agent(self, agent):
         self.agents['ovs'] = agent
 
+    @property
+    def sriov_agent(self):
+        return self.agents['sriov']
+
+    @sriov_agent.setter
+    def sriov_agent(self, agent):
+        self.agents['sriov'] = agent
+
     @property
     def linuxbridge_agent(self):
         return self.agents['linuxbridge']
@@ -261,6 +328,15 @@ class Host(fixtures.Fixture):
     def linuxbridge_agent(self, agent):
         self.agents['linuxbridge'] = agent
 
+    @property
+    def l2_agent(self):
+        if self.host_desc.l2_agent_type == constants.AGENT_TYPE_LINUXBRIDGE:
+            return self.linuxbridge_agent
+        elif self.host_desc.l2_agent_type == constants.AGENT_TYPE_OVS:
+            return self.ovs_agent
+        elif self.host_desc.l2_agent_type == constants.AGENT_TYPE_NIC_SWITCH:
+            return self.sriov_agent
+
 
 class Environment(fixtures.Fixture):
     """Represents a deployment topology.
@@ -272,7 +348,8 @@ class Environment(fixtures.Fixture):
     """
 
     def __init__(self, env_desc, hosts_desc):
-        """
+        """Initialize Environment
+
         :param env_desc: An EnvironmentDescription instance.
         :param hosts_desc: A list of HostDescription instances.
         """
@@ -283,7 +360,10 @@ class Environment(fixtures.Fixture):
         self.hosts = []
 
     def wait_until_env_is_up(self):
-        common_utils.wait_until_true(self._processes_are_ready)
+        common_utils.wait_until_true(
+            self._processes_are_ready,
+            timeout=180,
+            sleep=10)
 
     def _processes_are_ready(self):
         try:
@@ -305,19 +385,16 @@ class Environment(fixtures.Fixture):
                  host_desc,
                  self.test_name,
                  neutron_config,
-                 self.central_data_bridge,
-                 self.central_external_bridge))
+                 self.central_bridge))
 
     def _setUp(self):
         self.temp_dir = self.useFixture(fixtures.TempDir()).path
 
-        #we need this bridge before rabbit and neutron service will start
-        self.central_data_bridge = self.useFixture(
+        # we need this bridge before rabbit and neutron service will start
+        self.central_bridge = self.useFixture(
             net_helpers.OVSBridgeFixture('cnt-data')).bridge
-        self.central_external_bridge = self.useFixture(
-            net_helpers.OVSBridgeFixture('cnt-ex')).bridge
 
-        #Get rabbitmq address (and cnt-data network)
+        # Get rabbitmq address (and cnt-data network)
         rabbitmq_ip_address = self._configure_port_for_rabbitmq()
         self.rabbitmq_environment = self.useFixture(
             process.RabbitmqEnvironmentFixture(host=rabbitmq_ip_address)
@@ -336,6 +413,17 @@ class Environment(fixtures.Fixture):
                 self.env_desc, None,
                 self.test_name, neutron_cfg_fixture, plugin_cfg_fixture))
 
+        if self.env_desc.has_placement:
+            placement_cfg_fixture = self.useFixture(
+                config.PlacementConfigFixture(self.env_desc, self.hosts_desc,
+                                              self.temp_dir)
+            )
+            self.placement = self.useFixture(
+                process.PlacementFixture(
+                    self.env_desc, self.hosts_desc, self.test_name,
+                    placement_cfg_fixture)
+            )
+
         self.hosts = [self._create_host(desc) for desc in self.hosts_desc]
 
         self.wait_until_env_is_up()
@@ -345,14 +433,14 @@ class Environment(fixtures.Fixture):
         if not self.env_desc.network_range:
             return "127.0.0.1"
         rabbitmq_ip = str(self.env_desc.network_range[1])
-        rabbitmq_port = ip_lib.IPDevice(self.central_data_bridge.br_name)
+        rabbitmq_port = ip_lib.IPDevice(self.central_bridge.br_name)
         rabbitmq_port.addr.add(common_utils.ip_to_cidr(rabbitmq_ip, 24))
         rabbitmq_port.link.set_up()
 
         return rabbitmq_ip
 
     def _get_network_range(self):
-        #NOTE(slaweq): We need to choose IP address on which rabbitmq will be
+        # NOTE(slaweq): We need to choose IP address on which rabbitmq will be
         # available because LinuxBridge agents are spawned in their own
         # namespaces and need to know where the rabbitmq server is listening.
         # For ovs agent it is not necessary because agents are spawned in
