@@ -12,11 +12,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
 from neutron_lib.callbacks import events as callbacks_events
 from neutron_lib.callbacks import registry as callbacks_registry
 from neutron_lib.callbacks import resources as callbacks_resources
 from neutron_lib import constants
+from neutron_lib.utils import helpers
+from oslo_config import cfg
 import testtools
 
 from neutron.agent.common import ovs_lib
@@ -24,20 +27,25 @@ from neutron.agent.common import utils
 from neutron.agent.linux.openvswitch_firewall import constants as ovsfw_consts
 from neutron.agent.linux.openvswitch_firewall import exceptions
 from neutron.agent.linux.openvswitch_firewall import firewall as ovsfw
-from neutron.common import constants as n_const
+from neutron.conf.agent import securitygroups_rpc
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants \
         as ovs_consts
-from neutron.plugins.ml2.drivers.openvswitch.agent.openflow.ovs_ofctl \
+from neutron.plugins.ml2.drivers.openvswitch.agent.openflow.native \
     import ovs_bridge
 from neutron.tests import base
 
 TESTING_VLAN_TAG = 1
+TESTING_SEGMENT = 1000
 
 
-def create_ofport(port_dict):
+def create_ofport(port_dict, network_type=None,
+                  physical_network=None, segment_id=TESTING_SEGMENT):
     ovs_port = mock.Mock(vif_mac='00:00:00:00:00:00', ofport=1,
                          port_name="port-name")
-    return ovsfw.OFPort(port_dict, ovs_port, vlan_tag=TESTING_VLAN_TAG)
+    return ovsfw.OFPort(port_dict, ovs_port, vlan_tag=TESTING_VLAN_TAG,
+                        segment_id=segment_id,
+                        network_type=network_type,
+                        physical_network=physical_network)
 
 
 class TestCreateRegNumbers(base.BaseTestCase):
@@ -275,23 +283,50 @@ class TestConjIdMap(base.BaseTestCase):
                           constants.IPv6)
 
     def test_delete_sg(self):
-        test_data = [('sg1', 'sg1'), ('sg1', 'sg2')]
+        test_data = [
+            # conj_id: 8
+            ('sg1', 'sg1', constants.INGRESS_DIRECTION, constants.IPv6, 0),
+            # conj_id: 10
+            ('sg1', 'sg1', constants.INGRESS_DIRECTION, constants.IPv6, 1),
+            # conj_id: 12
+            ('sg1', 'sg1', constants.INGRESS_DIRECTION, constants.IPv6, 2),
+            # conj_id: 16
+            ('sg2', 'sg1', constants.EGRESS_DIRECTION, constants.IPv6, 0),
+            # conj_id: 24
+            ('sg1', 'sg3', constants.INGRESS_DIRECTION, constants.IPv6, 0),
+            # conj_id: 36 (and 32 without priority offset, stored in id_map)
+            ('sg3', 'sg4', constants.INGRESS_DIRECTION, constants.IPv4, 2),
+            # conj_id: 40
+            ('sg5', 'sg4', constants.EGRESS_DIRECTION, constants.IPv4, 0),
+        ]
 
         ids = []
-        for sg_id, remote_sg_id in test_data:
-            ids.append(self.conj_id_map.get_conj_id(
-                sg_id, remote_sg_id,
-                constants.INGRESS_DIRECTION, constants.IPv6))
+        conj_id_segment = set([])  # see ConjIPFlowManager.get_conj_id
+        # This is similar to ConjIPFlowManager.add method
+        for sg_id, rsg_id, direction, ip_version, prio_offset in test_data:
+            conj_id_tuple = (sg_id, rsg_id, direction, ip_version)
+            conj_id = self.conj_id_map.get_conj_id(*conj_id_tuple)
+            conj_id_segment.add(conj_id)
+            conj_id_plus_prio = conj_id + prio_offset * 2
+            self.conj_id_map.id_map_group[conj_id_tuple].add(conj_id_plus_prio)
+            ids.append(conj_id_plus_prio)
 
         result = self.conj_id_map.delete_sg('sg1')
-        self.assertIn(('sg1', ids[0]), result)
-        self.assertIn(('sg2', ids[1]), result)
-        self.assertFalse(self.conj_id_map.id_map)
+        self.assertEqual(
+            {('sg3', 24), ('sg1', 12), ('sg1', 16), ('sg1', 8), ('sg1', 10)},
+            result)
+        result = self.conj_id_map.delete_sg('sg3')
+        self.assertEqual({('sg4', 32), ('sg4', 36)}, result)
+        result = self.conj_id_map.delete_sg('sg4')
+        self.assertEqual({('sg4', 40)}, result)
+        self.assertEqual({}, self.conj_id_map.id_map)
+        self.assertEqual({}, self.conj_id_map.id_map_group)
 
-        reallocated = self.conj_id_map.get_conj_id(
-            'sg-foo', 'sg-foo', constants.INGRESS_DIRECTION,
-            constants.IPv6)
-        self.assertIn(reallocated, ids)
+        reallocated = set([])
+        for sg_id, rsg_id, direction, ip_version, _ in test_data:
+            conj_id_tuple = (sg_id, rsg_id, direction, ip_version)
+            reallocated.add(self.conj_id_map.get_conj_id(*conj_id_tuple))
+        self.assertEqual(reallocated, conj_id_segment)
 
 
 class TestConjIPFlowManager(base.BaseTestCase):
@@ -302,10 +337,38 @@ class TestConjIPFlowManager(base.BaseTestCase):
         self.vlan_tag = 100
         self.conj_id = 16
 
+    def test_update_flows_for_vlan_no_members(self):
+        remote_group = self.driver.sg_port_map.get_sg.return_value
+        remote_group.members = {}
+        with mock.patch.object(self.manager.conj_id_map,
+                               'get_conj_id') as get_conj_id_mock:
+            get_conj_id_mock.return_value = self.conj_id
+            self.manager.add(self.vlan_tag, 'sg', 'remote_id',
+                             constants.INGRESS_DIRECTION, constants.IPv4, 0)
+            self.manager.update_flows_for_vlan(self.vlan_tag)
+        self.assertFalse(remote_group.get_ethertype_filtered_addresses.called)
+        self.assertFalse(self.driver._add_flow.called)
+
+    def test_update_flows_for_vlan_no_ports_but_members(self):
+        remote_group = self.driver.sg_port_map.get_sg.return_value
+        remote_group.ports = set()
+        remote_group.members = {constants.IPv4: [
+            ('10.22.3.4', 'fa:16:3e:aa:bb:cc'), ]}
+        remote_group.get_ethertype_filtered_addresses.return_value = [
+            ('10.22.3.4', 'fa:16:3e:aa:bb:cc'), ]
+        with mock.patch.object(self.manager.conj_id_map,
+                               'get_conj_id') as get_conj_id_mock:
+            get_conj_id_mock.return_value = self.conj_id
+            self.manager.add(self.vlan_tag, 'sg', 'remote_id',
+                             constants.INGRESS_DIRECTION, constants.IPv4, 0)
+            self.manager.update_flows_for_vlan(self.vlan_tag)
+        self.assertTrue(remote_group.get_ethertype_filtered_addresses.called)
+        self.assertTrue(self.driver._add_flow.called)
+
     def test_update_flows_for_vlan(self):
         remote_group = self.driver.sg_port_map.get_sg.return_value
         remote_group.get_ethertype_filtered_addresses.return_value = [
-            '10.22.3.4']
+            ('10.22.3.4', 'fa:16:3e:aa:bb:cc'), ]
         with mock.patch.object(self.manager.conj_id_map,
                                'get_conj_id') as get_conj_id_mock:
             get_conj_id_mock.return_value = self.conj_id
@@ -328,7 +391,7 @@ class TestConjIPFlowManager(base.BaseTestCase):
                        dl_type=2048, nw_src='10.22.3.4/32', priority=73,
                        reg_net=self.vlan_tag, table=82)])
 
-    def test_sg_removed(self):
+    def _sg_removed(self, sg_name):
         with mock.patch.object(self.manager.conj_id_map,
                                'get_conj_id') as get_id_mock, \
              mock.patch.object(self.manager.conj_id_map,
@@ -341,11 +404,26 @@ class TestConjIPFlowManager(base.BaseTestCase):
                 constants.INGRESS_DIRECTION, constants.IPv4)] = {
                     '10.22.3.4': [self.conj_id]}
 
-            self.manager.sg_removed('sg')
+            self.manager.sg_removed(sg_name)
+
+    def test_sg_removed(self):
+        self._sg_removed('sg')
         self.driver._add_flow.assert_not_called()
-        self.driver.delete_flows_for_ip_addresses.assert_called_once_with(
-            {'10.22.3.4'}, constants.INGRESS_DIRECTION, constants.IPv4,
-            self.vlan_tag)
+        self.driver.delete_flows_for_flow_state.assert_called_once_with(
+            {'10.22.3.4': [self.conj_id]}, {},
+            constants.INGRESS_DIRECTION, constants.IPv4, self.vlan_tag)
+        self.driver.delete_flow_for_ip.assert_not_called()
+
+    def test_remote_sg_removed(self):
+        self._sg_removed('remote_id')
+        self.driver._add_flow.assert_not_called()
+        self.driver.delete_flows_for_flow_state.assert_called_once_with(
+            {'10.22.3.4': [self.conj_id]}, {},
+            constants.INGRESS_DIRECTION, constants.IPv4, self.vlan_tag)
+        # "conj_id_to_remove" is populated with the remote_sg conj_id assigned,
+        # "_update_flows_for_vlan_subr" will call "delete_flow_for_ip".
+        self.driver.delete_flow_for_ip.assert_called_once_with(
+            '10.22.3.4', 'ingress', 'IPv4', 100, {self.conj_id})
 
 
 class FakeOVSPort(object):
@@ -360,6 +438,7 @@ class TestOVSFirewallDriver(base.BaseTestCase):
         super(TestOVSFirewallDriver, self).setUp()
         mock_bridge = mock.patch.object(
             ovs_lib, 'OVSBridge', autospec=True).start()
+        securitygroups_rpc.register_securitygroups_opts()
         self.firewall = ovsfw.OVSFirewallDriver(mock_bridge)
         self.mock_bridge = self.firewall.int_br
         self.mock_bridge.reset_mock()
@@ -474,6 +553,7 @@ class TestOVSFirewallDriver(base.BaseTestCase):
         self.mock_bridge.br.get_vif_port_by_id.return_value = \
             fake_ovs_port
         port = self.firewall.get_or_create_ofport(port_dict)
+        self.assertIn(of_port.id, self.firewall.sg_port_map.ports.keys())
         self.assertEqual(port.ofport, 2)
 
     def test_get_or_create_ofport_missing(self):
@@ -531,7 +611,7 @@ class TestOVSFirewallDriver(base.BaseTestCase):
             'output:{:d},resubmit(,{:d})'.format(
                 self.port_ofport,
                 ovs_consts.ACCEPTED_INGRESS_TRAFFIC_TABLE),
-            dl_type="0x{:04x}".format(n_const.ETHERTYPE_IP),
+            dl_type="0x{:04x}".format(constants.ETHERTYPE_IP),
             nw_proto=constants.PROTO_NUM_TCP,
             priority=77,
             reg5=self.port_ofport,
@@ -552,6 +632,168 @@ class TestOVSFirewallDriver(base.BaseTestCase):
             self.firewall.prepare_port_filter(port_dict)
         self.assertFalse(m_init_flows.called)
 
+    def _test_initialize_port_flows_dvr_conntrack_direct(self, network_type):
+        port_dict = {
+            'device': 'port-id',
+            'security_groups': [1]}
+        segment_id = None
+        if network_type == constants.TYPE_VLAN:
+            segment_id = TESTING_SEGMENT
+        of_port = create_ofport(port_dict,
+                                network_type=network_type,
+                                segment_id=segment_id)
+        self.firewall.sg_port_map.ports[of_port.id] = of_port
+        port = self.firewall.get_or_create_ofport(port_dict)
+
+        fake_patch_port = 999
+        self.mock_bridge.br.get_port_ofport.return_value = fake_patch_port
+
+        expected_calls = []
+        self.firewall.initialize_port_flows(port)
+
+        call_args1 = {
+            'table': ovs_consts.TRANSIENT_TABLE,
+            'priority': 100,
+            'in_port': port.ofport,
+            'actions': 'set_field:{:d}->reg{:d},'
+                       'set_field:{:d}->reg{:d},'
+                       'resubmit(,{:d})'.format(
+                           port.ofport,
+                           ovsfw_consts.REG_PORT,
+                           port.vlan_tag,
+                           ovsfw_consts.REG_NET,
+                           ovs_consts.BASE_EGRESS_TABLE)}
+        expected_calls.append(mock.call(**call_args1))
+
+        if network_type == constants.TYPE_VLAN:
+            call_args2 = {
+                'table': ovs_consts.TRANSIENT_TABLE,
+                'priority': 90,
+                'dl_dst': port.mac,
+                'dl_vlan': '0x%x' % port.segment_id,
+                'actions': 'set_field:{:d}->reg{:d},'
+                           'set_field:{:d}->reg{:d},'
+                           'strip_vlan,resubmit(,{:d})'.format(
+                               port.ofport,
+                               ovsfw_consts.REG_PORT,
+                               port.vlan_tag,
+                               ovsfw_consts.REG_NET,
+                               ovs_consts.BASE_INGRESS_TABLE)}
+            expected_calls.append(mock.call(**call_args2))
+
+        if network_type == constants.TYPE_FLAT:
+            call_args2 = {
+                'table': ovs_consts.TRANSIENT_TABLE,
+                'priority': 90,
+                'dl_dst': port.mac,
+                'vlan_tci': ovs_consts.FLAT_VLAN_TCI,
+                'actions': 'set_field:{:d}->reg{:d},'
+                           'set_field:{:d}->reg{:d},'
+                           'resubmit(,{:d})'.format(
+                               port.ofport,
+                               ovsfw_consts.REG_PORT,
+                               port.vlan_tag,
+                               ovsfw_consts.REG_NET,
+                               ovs_consts.BASE_INGRESS_TABLE)}
+            expected_calls.append(mock.call(**call_args2))
+
+        call_args3 = {
+            'table': ovs_consts.TRANSIENT_TABLE,
+            'priority': 90,
+            'dl_dst': port.mac,
+            'dl_vlan': '0x%x' % port.vlan_tag,
+            'actions': 'set_field:{:d}->reg{:d},'
+                       'set_field:{:d}->reg{:d},'
+                       'strip_vlan,resubmit(,{:d})'.format(
+                           port.ofport,
+                           ovsfw_consts.REG_PORT,
+                           port.vlan_tag,
+                           ovsfw_consts.REG_NET,
+                           ovs_consts.BASE_INGRESS_TABLE)}
+        expected_calls.append(mock.call(**call_args3))
+        self.mock_bridge.br.add_flow.assert_has_calls(expected_calls)
+
+    def test_initialize_port_flows_dvr_conntrack_direct_vxlan(self):
+        self._test_initialize_port_flows_dvr_conntrack_direct(
+                network_type='vxlan')
+
+    def test_initialize_port_flows_dvr_conntrack_direct_vlan(self):
+        self._test_initialize_port_flows_dvr_conntrack_direct(
+                network_type='vlan')
+
+    def test_initialize_port_flows_dvr_conntrack_direct_flat(self):
+        self._test_initialize_port_flows_dvr_conntrack_direct(
+                network_type='flat')
+
+    def test_initialize_port_flows_vlan_dvr_conntrack_direct_vlan(self):
+        port_dict = {
+            'device': 'port-id',
+            'security_groups': [1]}
+        of_port = create_ofport(port_dict,
+                                network_type=constants.TYPE_VLAN,
+                                physical_network='vlan1')
+        self.firewall.sg_port_map.ports[of_port.id] = of_port
+        port = self.firewall.get_or_create_ofport(port_dict)
+
+        fake_patch_port = 999
+        self.mock_bridge.br.get_port_ofport.return_value = fake_patch_port
+
+        with mock.patch.object(helpers, "parse_mappings",
+                               return_value={"vlan1": "br-vlan1"}):
+            self.firewall.initialize_port_flows(port)
+
+    def test_delete_all_port_flows(self):
+        port_dict = {
+            'device': 'port-id',
+            'security_groups': [1]}
+        of_port = create_ofport(port_dict,
+                                network_type=constants.TYPE_VXLAN)
+        self.firewall.sg_port_map.ports[of_port.id] = of_port
+        port = self.firewall.get_or_create_ofport(port_dict)
+
+        self.firewall.delete_all_port_flows(port)
+
+        call_args1 = {"strict": True,
+                      "priority": 90,
+                      "table": ovs_consts.TRANSIENT_TABLE,
+                      "dl_dst": port.mac,
+                      "dl_vlan": port.vlan_tag}
+        flow1 = mock.call(**call_args1)
+
+        call_args2 = {"strict": True,
+                      "priority": 90,
+                      "table": ovs_consts.TRANSIENT_TABLE,
+                      "dl_dst": port.mac,
+                      "dl_vlan": port.segment_id}
+        flow2 = mock.call(**call_args2)
+
+        call_args3 = {"table": ovs_consts.ACCEPT_OR_INGRESS_TABLE,
+                      "dl_dst": port.mac,
+                      "reg6": port.vlan_tag}
+        flow3 = mock.call(**call_args3)
+
+        call_args4 = {"in_port": port.ofport,
+                      "strict": True,
+                      "priority": 100,
+                      "table": ovs_consts.TRANSIENT_TABLE}
+        flow4 = mock.call(**call_args4)
+
+        call_args5 = {"reg5": port.ofport}
+        flow5 = mock.call(**call_args5)
+
+        call_args6 = {"table": ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE,
+                      "dl_dst": port.mac,
+                      "reg6": port.vlan_tag}
+        flow6 = mock.call(**call_args6)
+
+        call_args7 = {"table": ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE,
+                      "dl_src": port.mac,
+                      "reg6": port.vlan_tag}
+        flow7 = mock.call(**call_args7)
+
+        self.mock_bridge.br.delete_flows.assert_has_calls(
+            [flow1, flow2, flow3, flow6, flow7, flow4, flow5])
+
     def test_prepare_port_filter_initialized_port(self):
         port_dict = {'device': 'port-id',
                      'security_groups': [1]}
@@ -576,7 +818,7 @@ class TestOVSFirewallDriver(base.BaseTestCase):
         filter_rules = [mock.call(
             actions='resubmit(,{:d})'.format(
                 ovs_consts.ACCEPT_OR_INGRESS_TABLE),
-            dl_type="0x{:04x}".format(n_const.ETHERTYPE_IP),
+            dl_type="0x{:04x}".format(constants.ETHERTYPE_IP),
             nw_proto=constants.PROTO_NUM_UDP,
             priority=77,
             ct_state=ovsfw_consts.OF_STATE_NEW_NOT_ESTABLISHED,
@@ -630,6 +872,16 @@ class TestOVSFirewallDriver(base.BaseTestCase):
             self.firewall.update_port_filter(port_dict)
         self.assertEqual(2, self.mock_bridge.apply_flows.call_count)
 
+    def test_update_port_filter_clean_when_port_not_found(self):
+        """Check flows are cleaned if port is not found in the bridge."""
+        port_dict = {'device': 'port-id',
+                     'security_groups': [1]}
+        self._prepare_security_group()
+        self.firewall.prepare_port_filter(port_dict)
+        self.mock_bridge.br.get_vif_port_by_id.return_value = None
+        self.firewall.update_port_filter(port_dict)
+        self.assertTrue(self.mock_bridge.br.delete_flows.called)
+
     def test_remove_port_filter(self):
         port_dict = {'device': 'port-id',
                      'security_groups': [1]}
@@ -681,6 +933,14 @@ class TestOVSFirewallDriver(base.BaseTestCase):
         with testtools.ExpectedException(exceptions.OVSFWPortNotFound):
             self.firewall.get_ovs_port('port_id')
 
+    def test_get_ovs_port_invalid(self):
+        vif_port = ovs_lib.VifPort('name', 'ofport', 'id', 'mac', 'switch')
+        self.mock_bridge.br.get_vif_port_by_id.return_value = vif_port
+        for ofport in (ovs_lib.UNASSIGNED_OFPORT, ovs_lib.INVALID_OFPORT):
+            vif_port.ofport = ofport
+            with testtools.ExpectedException(exceptions.OVSFWPortNotFound):
+                self.firewall.get_ovs_port('port_id')
+
     def test__initialize_egress_no_port_security_sends_to_egress(self):
         self.mock_bridge.br.db_get_val.return_value = {'tag': TESTING_VLAN_TAG}
         self.firewall._initialize_egress_no_port_security('port_id')
@@ -707,7 +967,8 @@ class TestOVSFirewallDriver(base.BaseTestCase):
 
     def test__remove_egress_no_port_security_deletes_flow(self):
         self.mock_bridge.br.db_get_val.return_value = {'tag': TESTING_VLAN_TAG}
-        self.firewall.sg_port_map.unfiltered['port_id'] = 1
+        self.firewall.sg_port_map.unfiltered['port_id'] = (
+            self.fake_ovs_port, 100)
         self.firewall._remove_egress_no_port_security('port_id')
         expected_call = mock.call(
             table=ovs_consts.TRANSIENT_TABLE,
@@ -721,18 +982,26 @@ class TestOVSFirewallDriver(base.BaseTestCase):
             self.firewall._remove_egress_no_port_security('foo')
 
     def test_process_trusted_ports_caches_port_id(self):
-        self.firewall.process_trusted_ports(['port_id'])
-        self.assertIn('port_id', self.firewall.sg_port_map.unfiltered)
+        vif_port = ovs_lib.VifPort('name', 1, 'id', 'mac', mock.ANY)
+        with mock.patch.object(self.firewall.int_br.br, 'get_vifs_by_ids',
+                               return_value={'port_id': vif_port}):
+            self.firewall.process_trusted_ports(['port_id'])
+            self.assertEqual(1,
+                             len(self.firewall.sg_port_map.unfiltered.keys()))
+            ofport, _ = self.firewall.sg_port_map.unfiltered['port_id']
+            self.assertEqual(vif_port.ofport, ofport.ofport)
 
     def test_process_trusted_ports_port_not_found(self):
         """Check that exception is not propagated outside."""
-        self.mock_bridge.br.get_vif_port_by_id.return_value = None
-        self.firewall.process_trusted_ports(['port_id'])
-        # Processing should have failed so port is not cached
-        self.assertNotIn('port_id', self.firewall.sg_port_map.unfiltered)
+        with mock.patch.object(self.firewall.int_br.br, 'get_vifs_by_ids',
+                               return_value={}):
+            self.firewall.process_trusted_ports(['port_id'])
+            # Processing should have failed so port is not cached
+            self.assertEqual(0, len(self.firewall.sg_port_map.unfiltered))
 
     def test_remove_trusted_ports_clears_cached_port_id(self):
-        self.firewall.sg_port_map.unfiltered['port_id'] = 1
+        self.firewall.sg_port_map.unfiltered['port_id'] = (
+            self.fake_ovs_port, 100)
         self.firewall.remove_trusted_ports(['port_id'])
         self.assertNotIn('port_id', self.firewall.sg_port_map.unfiltered)
 
@@ -740,6 +1009,53 @@ class TestOVSFirewallDriver(base.BaseTestCase):
         """Check that exception is not propagated outside."""
         self.firewall.remove_trusted_ports(['port_id'])
 
+    def _test_delete_flows_for_flow_state(self, addr_to_conj,
+                                          explicitly_egress_direct=True):
+        direction = 'one_direction'
+        ethertype = 'ethertype'
+        vlan_tag = 'taaag'
+        with mock.patch.object(self.firewall, 'delete_flow_for_ip') as \
+                mock_delete_flow_for_ip:
+            flow_state = {'addr1': {8, 16, 24}, 'addr2': {32, 40}}
+            cfg.CONF.set_override('explicitly_egress_direct',
+                                  explicitly_egress_direct, 'AGENT')
+            self.firewall.delete_flows_for_flow_state(
+                flow_state, addr_to_conj, direction, ethertype, vlan_tag)
+        calls = []
+        for removed_ip in flow_state.keys() - addr_to_conj.keys():
+            calls.append(mock.call(removed_ip, direction, ethertype, vlan_tag,
+                                   flow_state[removed_ip]))
+            if explicitly_egress_direct:
+                calls.append(mock.call(removed_ip, direction, ethertype,
+                                       vlan_tag, [0]))
+        mock_delete_flow_for_ip.assert_has_calls(calls)
+
+    def test_delete_flows_for_flow_state_no_removed_ips_exp_egress(self):
+        addr_to_conj = {'addr1': {8, 16, 24}, 'addr2': {32, 40}}
+        self._test_delete_flows_for_flow_state(addr_to_conj)
+
+    def test_delete_flows_for_flow_state_no_removed_ips_no_exp_egress(self):
+        addr_to_conj = {'addr1': {8, 16, 24}, 'addr2': {32, 40}}
+        self._test_delete_flows_for_flow_state(addr_to_conj, False)
+
+    def test_delete_flows_for_flow_state_removed_ips_exp_egress(self):
+        addr_to_conj = {'addr2': {32, 40}}
+        self._test_delete_flows_for_flow_state(addr_to_conj)
+
+    def test_delete_flows_for_flow_state_removed_ips_no_exp_egress(self):
+        addr_to_conj = {'addr1': {8, 16, 24}}
+        self._test_delete_flows_for_flow_state(addr_to_conj, False)
+
+    def test_delete_flow_for_ip_using_cookie_any(self):
+        with mock.patch.object(self.firewall, '_delete_flows') as \
+                mock_delete_flows:
+            self.firewall.delete_flow_for_ip(('10.1.2.3', None),
+                                             constants.INGRESS_DIRECTION,
+                                             constants.IPv4, 100, [0])
+            _, kwargs = mock_delete_flows.call_args
+            self.assertIn('cookie', kwargs)
+            self.assertIs(ovs_lib.COOKIE_ANY, kwargs['cookie'])
+
 
 class TestCookieContext(base.BaseTestCase):
     def setUp(self):
@@ -752,12 +1068,13 @@ class TestCookieContext(base.BaseTestCase):
 
         self.execute = mock.patch.object(
             utils, "execute", spec=utils.execute).start()
-        bridge = ovs_bridge.OVSAgentBridge('foo')
+        bridge = ovs_bridge.OVSAgentBridge('foo', os_ken_app=mock.Mock())
         mock.patch.object(
             ovsfw.OVSFirewallDriver, 'initialize_bridge',
             return_value=bridge.deferred(
                 full_ordered=True, use_bundle=True)).start()
 
+        securitygroups_rpc.register_securitygroups_opts()
         self.firewall = ovsfw.OVSFirewallDriver(bridge)
         # Remove calls from firewall initialization
         self.execute.reset_mock()
